@@ -17,7 +17,6 @@
 
 #include <config.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
 
@@ -51,7 +50,8 @@ Usage: %s NUMBER[SUFFIX]...\n\
 Pause for NUMBER seconds.  SUFFIX may be `s' for seconds (the default),\n\
 `m' for minutes, `h' for hours or `d' for days.  Unlike most implementations\n\
 that require NUMBER be an integer, here NUMBER may be an arbitrary floating\n\
-point number.\n\
+point number.  Given two or more arguments, pause for the amount of time\n\
+specified by the sum of their values.\n\
 \n\
 "),
 	      program_name, program_name);
