@@ -55,7 +55,8 @@ you must include <sys/types.h> before including this file
 #include <limits.h>
 
 #include "pathmax.h"
-#include "localedir.h"
+
+#include "configmake.h"
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -96,17 +97,23 @@ you must include <sys/types.h> before including this file
    memcpy otherwise.  */
 
 #include <string.h>
+#include "mempcpy.h"
 #include "memrchr.h"
+#include "stpcpy.h"
+#include "strpbrk.h"
 
 #include <errno.h>
 
 /* Some systems don't define the following symbols.  */
-#ifndef ENOSYS
-# define ENOSYS (-1)
+#ifndef EDQUOT
+# define EDQUOT (-1)
 #endif
 #ifndef EISDIR
 # define EISDIR (-1)
 #endif
+#ifndef ENOSYS
+# define ENOSYS (-1)
+#endif
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -144,11 +151,6 @@ initialize_exit_failure (int status)
 
 #include <fcntl.h>
 
-#if !defined SEEK_SET
-# define SEEK_SET 0
-# define SEEK_CUR 1
-# define SEEK_END 2
-#endif
 #ifndef F_OK
 # define F_OK 0
 # define X_OK 1
@@ -156,74 +158,6 @@ initialize_exit_failure (int status)
 # define R_OK 4
 #endif
 
-#if !defined O_DIRECT && defined O_DIRECTIO
-/* Tru64 spells it `O_DIRECTIO'.  */
-# define O_DIRECT O_DIRECTIO
-#endif
-
-#if !defined O_DIRECT
-# define O_DIRECT 0
-#endif
-
-#if !defined O_DIRECTORY
-# define O_DIRECTORY 0
-#endif
-
-#if !defined O_DSYNC
-# define O_DSYNC 0
-#endif
-
-#if !defined O_NDELAY
-# define O_NDELAY 0
-#endif
-
-#if !defined O_NOATIME
-# define O_NOATIME 0
-#endif
-
-#if !defined O_NONBLOCK
-# define O_NONBLOCK O_NDELAY
-#endif
-
-#if !defined O_NOCTTY
-# define O_NOCTTY 0
-#endif
-
-#if !defined O_NOFOLLOW
-# define O_NOFOLLOW 0
-#endif
-
-#if !defined O_NOLINKS
-# define O_NOLINKS 0
-#endif
-
-#if !defined O_RSYNC
-# define O_RSYNC 0
-#endif
-
-#if !defined O_SYNC
-# define O_SYNC 0
-#endif
-
-/* For systems that distinguish between text and binary I/O.
-   O_BINARY is usually declared in fcntl.h  */
-#if !defined O_BINARY && defined _O_BINARY
-  /* For MSC-compatible compilers.  */
-# define O_BINARY _O_BINARY
-# define O_TEXT _O_TEXT
-#endif
-
-#ifdef __BEOS__
-  /* BeOS 5 has O_BINARY and O_TEXT, but they have no effect.  */
-# undef O_BINARY
-# undef O_TEXT
-#endif
-
-#ifndef O_BINARY
-# define O_BINARY 0
-# define O_TEXT 0
-#endif
-
 #include <dirent.h>
 #ifndef _D_EXACT_NAMLEN
 # define _D_EXACT_NAMLEN(dp) strlen ((dp)->d_name)
@@ -319,34 +253,7 @@ enum
 
 #include "timespec.h"
 
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
-#include <stdint.h>
-
-#if ULONG_MAX_LT_ULLONG_MAX
-# define LONGEST_MODIFIER "ll"
-#else
-# define LONGEST_MODIFIER "l"
-#endif
-#if PRI_MACROS_BROKEN
-# undef PRIdMAX
-# undef PRIoMAX
-# undef PRIuMAX
-# undef PRIxMAX
-#endif
-#ifndef PRIdMAX
-# define PRIdMAX LONGEST_MODIFIER "d"
-#endif
-#ifndef PRIoMAX
-# define PRIoMAX LONGEST_MODIFIER "o"
-#endif
-#ifndef PRIuMAX
-# define PRIuMAX LONGEST_MODIFIER "u"
-#endif
-#ifndef PRIxMAX
-# define PRIxMAX LONGEST_MODIFIER "x"
-#endif
+#include <inttypes.h>
 
 #include <ctype.h>
 
@@ -412,20 +319,6 @@ char *memchr ();
 char *realloc ();
 #endif
 
-#if !HAVE_DECL_STPCPY
-# ifndef stpcpy
-char *stpcpy ();
-# endif
-#endif
-
-#if !HAVE_DECL_STRNDUP
-char *strndup ();
-#endif
-
-#if !HAVE_DECL_STRSTR
-char *strstr ();
-#endif
-
 #if !HAVE_DECL_GETENV
 char *getenv ();
 #endif
@@ -434,11 +327,6 @@ char *getenv ();
 off_t lseek ();
 #endif
 
-/* This is needed on some AIX systems.  */
-#if !HAVE_DECL_STRTOUL
-unsigned long strtoul ();
-#endif
-
 #if !HAVE_DECL_GETLOGIN
 char *getlogin ();
 #endif
@@ -480,18 +368,22 @@ uid_t getuid ();
 #define X2REALLOC(P, PN) ((void) verify_true (sizeof *(P) == 1), \
                           x2realloc (P, PN))
 
-#if ! defined HAVE_MEMPCPY && ! defined mempcpy
-/* Be CAREFUL that there are no side effects in N.  */
-# define mempcpy(D, S, N) ((void *) ((char *) memcpy (D, S, N) + (N)))
-#endif
-
-/* Include automatically-generated macros for unlocked I/O.  */
 #include "unlocked-io.h"
 #include "same-inode.h"
 
-#define DOT_OR_DOTDOT(Basename) \
-  (Basename[0] == '.' && (Basename[1] == '\0' \
-			  || (Basename[1] == '.' && Basename[2] == '\0')))
+#include "dirname.h"
+
+static inline bool
+dot_or_dotdot (char const *file_name)
+{
+  if (file_name[0] == '.')
+    {
+      char sep = file_name[(file_name[1] == '.') + 1];
+      return (! sep || ISSLASH (sep));
+    }
+  else
+    return false;
+}
 
 /* A wrapper for readdir so that callers don't see entries for `.' or `..'.  */
 static inline struct dirent const *
@@ -500,7 +392,7 @@ readdir_ignoring_dot_and_dotdot (DIR *dirp)
   while (1)
     {
       struct dirent const *dp = readdir (dirp);
-      if (dp == NULL || ! DOT_OR_DOTDOT (dp->d_name))
+      if (dp == NULL || ! dot_or_dotdot (dp->d_name))
 	return dp;
     }
 }
@@ -567,74 +459,10 @@ enum
 
 #include "intprops.h"
 
-#ifndef CHAR_MIN
-# define CHAR_MIN TYPE_MINIMUM (char)
-#endif
-
-#ifndef CHAR_MAX
-# define CHAR_MAX TYPE_MAXIMUM (char)
-#endif
-
-#ifndef SCHAR_MIN
-# define SCHAR_MIN (-1 - SCHAR_MAX)
-#endif
-
-#ifndef SCHAR_MAX
-# define SCHAR_MAX (CHAR_MAX == UCHAR_MAX ? CHAR_MAX / 2 : CHAR_MAX)
-#endif
-
-#ifndef UCHAR_MAX
-# define UCHAR_MAX TYPE_MAXIMUM (unsigned char)
-#endif
-
-#ifndef SHRT_MIN
-# define SHRT_MIN TYPE_MINIMUM (short int)
-#endif
-
-#ifndef SHRT_MAX
-# define SHRT_MAX TYPE_MAXIMUM (short int)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX TYPE_MAXIMUM (int)
-#endif
-
-#ifndef INT_MIN
-# define INT_MIN TYPE_MINIMUM (int)
-#endif
-
-#ifndef INTMAX_MAX
-# define INTMAX_MAX TYPE_MAXIMUM (intmax_t)
-#endif
-
-#ifndef INTMAX_MIN
-# define INTMAX_MIN TYPE_MINIMUM (intmax_t)
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX TYPE_MAXIMUM (unsigned int)
-#endif
-
-#ifndef LONG_MAX
-# define LONG_MAX TYPE_MAXIMUM (long int)
-#endif
-
-#ifndef ULONG_MAX
-# define ULONG_MAX TYPE_MAXIMUM (unsigned long int)
-#endif
-
-#ifndef SIZE_MAX
-# define SIZE_MAX TYPE_MAXIMUM (size_t)
-#endif
-
 #ifndef SSIZE_MAX
 # define SSIZE_MAX TYPE_MAXIMUM (ssize_t)
 #endif
 
-#ifndef UINTMAX_MAX
-# define UINTMAX_MAX TYPE_MAXIMUM (uintmax_t)
-#endif
-
 #ifndef OFF_T_MIN
 # define OFF_T_MIN TYPE_MINIMUM (off_t)
 #endif
